"""A small sphinx extension to remove toctrees."""

from pathlib import Path
from sphinx.util import logging
from sphinx import addnodes


__version__ = "1.0.0"

logger = logging.getLogger(__name__)


def findall(node):
    # findall replaces traverse in docutils v0.18
    # note a difference is that findall is an iterator
    return getattr(node, "findall", node.traverse)


def remove_toctrees(app, env):
    """Remove toctrees from pages a user provides.

    This happens at the end of the build process, so even though the toctrees
    are removed, it won't raise sphinx warnings about un-referenced pages.
    """
    if len(app.config.remove_toctrees_from) > 0:
        app.config.remove_from_toctrees = app.config.remove_toctrees_from
        logger.warning(
            "`remove_toctrees_from` is deprecated, use `remove_from_toctrees`"
        )
    patterns = app.config.remove_from_toctrees
    if isinstance(patterns, str):
        patterns = [patterns]

    # Figure out the list of patterns to remove from all toctrees
    to_remove = []
    for pattern in patterns:
        # Inputs should either be a glob pattern or a direct path so just use glob
        srcdir = Path(env.srcdir)
        for matched in srcdir.glob(pattern):
            to_remove.append(
                str(matched.relative_to(srcdir).with_suffix("").as_posix())
            )

    # Loop through all tocs and remove the ones that match our pattern
    for _, tocs in env.tocs.items():
        for toctree in findall(tocs)(addnodes.toctree):
            new_entries = []
            for entry in toctree.attributes.get("entries", []):
                if entry[1] not in to_remove:
                    new_entries.append(entry)
            # If there are no more entries just remove the toctree
            if len(new_entries) == 0:
                toctree.parent.remove(toctree)
            else:
                toctree.attributes["entries"] = new_entries


def setup(app):  # noqa: D103
    app.add_config_value("remove_toctrees_from", [], "html")
    app.add_config_value("remove_from_toctrees", [], "html")
    app.connect("env-updated", remove_toctrees)
    return {"parallel_read_safe": True, "parallel_write_safe": True}



import os
from pathlib import Path
from shutil import copytree

from bs4 import BeautifulSoup
from sphinx import version_info as sphinx_version_info

pytest_plugins = "sphinx.testing.fixtures"

path_test_doc = Path(__file__).parent / "site"


def test_build_html(make_app, tmp_path):
    """Test building the base html template and config."""
    src_dir = tmp_path / "test_doc"
    copytree(path_test_doc, src_dir)    

    # For compatibility with multiple versions of sphinx, convert pathlib.Path to
    # sphinx.testing.path.path here.
    if sphinx_version_info >= (7, 2):
        app_src_dir = src_dir
    else:
        from sphinx.testing.path import path

        app_src_dir = path(os.fspath(src_dir))

    app = make_app(srcdir=app_src_dir)
    app.build()
    index = tmp_path / "test_doc" / "_build" / "html" / "index.html"
    assert index.exists()
    index = BeautifulSoup(index.read_text())
    sidenav = index.select("ul.bd-sidenav")[0]

    # Grab all references to second-level links, we should *only* find the shown page
    second_level_links = sidenav.select(".toctree-l2 a")
    assert len(second_level_links) == 1
    assert second_level_links[0].attrs["href"] == "nested/nested_page_shown.html"



import nox
from pathlib import Path

nox.options.reuse_existing_virtualenvs = True


@nox.session()
def docs(session):
    _install_environment(session)
    session.run("sphinx-build", "-b", "html", "docs", "docs/_build/html")


@nox.session(name="tests")
def tests(session):
    _install_environment(session)
    session.run("pytest", *session.posargs)


def _install_environment(session):
    """Install the JS and Python environment needed to develop the theme."""
    # Assume that if sphinx is already installed, we don't need to re-install
    bin = Path(session.bin)
    if list(bin.rglob("sphinx-build")) and "reinstall" not in session.posargs:
        return

    session.install("-e", ".[docs,tests]")



project = "Sphinx Remove Toctrees test"
extensions = ["sphinx_remove_toctrees", "myst_parser"]
master_doc = "index"
html_theme = "sphinx_book_theme"
remove_from_toctrees = ["nested/nested_page_hidden.md", "nested_hidden/*"]
